/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.isobus.model.util;

import java.math.BigDecimal;
import java.math.MathContext;

/**
 *
 * @author johan
 */
public class DatatypeConverters {
    
    public static final int GRID_CELL_SIZE_DECIMALS = 8;
    public static final int GRID_POSITION_DECIMALS = 8;
    
    public static String printGridCellSize(double d) {
        // round after converting to bigdecimal; a rounded double might add extra decimals during conversion since it's not precise.
        BigDecimal bd = limitDecimalPlaces(new BigDecimal(d), GRID_CELL_SIZE_DECIMALS);
        // write to xml without exponents:
        return bd.toPlainString();
    }
    
    public static String printGridPosition(BigDecimal d) {
        BigDecimal bd = limitDecimalPlaces(d, GRID_POSITION_DECIMALS);
        // write to xml without exponents:
        return bd.toPlainString();
    }
    
    protected static double limitDecimalPlaces(double value, int numberOfDecimals) {
        final double pow10 = Math.pow(10, numberOfDecimals);
        final double shiftLeft = value * pow10;
        assert shiftLeft <= Long.MAX_VALUE && shiftLeft >= Long.MIN_VALUE;
        return Math.round(shiftLeft) / pow10;
    }

    protected static BigDecimal limitDecimalPlaces(BigDecimal value, int numberOfDecimals) {
        assert numberOfDecimals >= 0;
        if (value.scale() > numberOfDecimals) {
            final int precision = value.precision() - (value.scale() - numberOfDecimals);
            assert precision > 0;
            final MathContext mc = new MathContext(precision);
            return value.round(mc);
        }
        return value;
    }
}
